@@ -2,14 +2,17 @@ package DBIx::QuickDB::Driver;
 use strict;
 use warnings;
 
-our $VERSION = '0.000010';
+our $VERSION = '0.000011';
 
 use Carp qw/croak confess/;
 use File::Path qw/remove_tree/;
+use File::Temp qw/tempdir/;
 use POSIX ":sys_wait_h";
 use Scalar::Util qw/blessed/;
 use Time::HiRes qw/sleep/;
 
+use DBIx::QuickDB::Util qw/clone_dir/;
+
 use DBIx::QuickDB::Util::HashBase qw{
     pid -root_pid log_file
     -dir
@@ -22,7 +25,7 @@ use DBIx::QuickDB::Util::HashBase qw{
     env_vars
 };
 
-sub viable { (0, "socket() is not implemented for the " . $_[0]->name . " driver") }
+sub viable { (0, "viable() is not implemented for the " . $_[0]->name . " driver") }
 
 sub socket         { confess "socket() is not implemented for the " . $_[0]->name . " driver" }
 sub load_sql       { confess "load_sql() is not implemented for the " . $_[0]->name . " driver" }
@@ -33,6 +36,10 @@ sub shell_command  { confess "shell_command() is not implemented for the " . $_[
 
 sub list_env_vars { qw/DBI_USER DBI_PASS DBI_DSN/ }
 
+sub version_string { 'unknown' }
+
+sub write_config {}
+
 sub do_in_env {
     my $self = shift;
     my ($code) = @_;
@@ -111,6 +118,60 @@ sub init {
     return;
 }
 
+sub clone_data {
+    my $self = shift;
+
+    return (
+        USERNAME()  => $self->{+USERNAME},
+        PASSWORD()  => $self->{+PASSWORD},
+        VERBOSE()   => $self->{+VERBOSE},
+        AUTOSTOP()  => $self->{+AUTOSTOP},
+        AUTOSTART() => $self->{+AUTOSTART},
+
+        cleanup => $self->{+_CLEANUP},
+
+        ENV_VARS() => {%{$self->{+ENV_VARS}}},
+    );
+}
+
+
+sub clone {
+    my $self = shift;
+    my %params = @_;
+
+    confess "Cannot clone a started database, please stop it first."
+        if $self->started;
+
+    my $orig_dir = $self->{+DIR};
+    my $new_dir  = delete $params{dir} // tempdir('DB-QUICK-CLONE-XXXXXX', CLEANUP => 0, TMPDIR => 1);
+
+    clone_dir($orig_dir, $new_dir, verbose => (($self->{+VERBOSE} // 0) > 2) ? 1 : 0);
+
+    my $class = ref($self);
+    my %ok = map {$_ => 1} DBIx::QuickDB::Util::HashBase::attr_list($class);
+    my @bad = grep { !$ok{$_} } keys %params;
+
+    confess "Invalid options to clone(): " . join(', ' => @bad)
+        if @bad;
+
+    my $clone = $class->new(
+        $self->clone_data,
+
+        %params,
+
+        DIR() => $new_dir,
+
+        PID()      => undef,
+        LOG_FILE() => undef,
+    );
+
+    $clone->write_config();
+    $clone->start if $clone->{+AUTOSTART};
+
+    return $clone;
+}
+
+
 sub run_command {
     my $self = shift;
     my ($cmd, $params) = @_;
@@ -182,6 +243,14 @@ sub connect {
     return $dbh;
 }
 
+sub started {
+    my $self = shift;
+
+    my $socket = $self->socket;
+    return 1 if $self->{+PID} || -S $socket;
+    return 0;
+}
+
 sub start {
     my $self = shift;
     my @args = @_;
@@ -223,13 +292,27 @@ sub start {
     return;
 }
 
+sub stop_sig { 'TERM' }
+
 sub stop {
     my $self = shift;
 
     my $pid = $self->{+PID} or return;
 
+    DBI->visit_handles(
+        sub {
+            my ($driver_handle) = @_;
+
+            $driver_handle->disconnect
+               if $driver_handle->{Type} && $driver_handle->{Type} eq 'db'
+               && $driver_handle->{Name} && index($driver_handle->{Name}, $self->{+DIR}) >= 0;
+
+            return 1;
+        }
+    );
+
     local $?;
-    kill('TERM', $pid);
+    kill($self->stop_sig, $pid);
     my $ret = waitpid($pid, 0);
     my $exit = $?;
     die "waitpid returned $ret (expected $pid)" unless $ret == $pid;
@@ -311,6 +394,7 @@ Base class for DBIx::QuickDB drivers.
 
     # Methods most drivers should implement
 
+    sub version_string { ... }
     sub socket         { ... }
     sub load_sql       { ... }
     sub bootstrap      { ... }
@@ -318,6 +402,10 @@ Base class for DBIx::QuickDB drivers.
     sub start_command  { ... }
     sub shell_command  { ... }
 
+    # Implement if necessary
+    sub write_config { ... }
+    sub stop_sig { return $SIG }
+
     1;
 
 =head1 METHODS PROVIDED HERE
@@ -464,6 +552,25 @@ Get/set the username to use in C<connect()>.
 
 If this is true then all output from C<run_command> will be shown at all times.
 
+=item $clone = $db->clone()
+
+=item $clone = $db->clone(%params)
+
+Create a copy of the database. This database should be identical, except it
+should not share any state changes moving forward, that means a new copy of all
+data, etc.
+
+=item %data = $db->clone_data()
+
+Data to use when cloning
+
+=item $db->write_config()
+
+no-op on the base class, used in cloning.
+
+=item $sig = $db->stop_sig()
+
+What signal to send to the database server to stop it. Default: C<'TERM'>.
 
 =item $db->DESTROY
 
@@ -538,6 +645,8 @@ the hashref returned by the first.
 
 =head1 METHODS SUBCLASSES SHOULD PROVIDE
 
+Drivers may override C<clone()> or C<clone_data()> to control cloning.
+
 =over
 
 =item ($bool, $why) = $db->viable()
@@ -553,6 +662,50 @@ The first return value is a simple boolean, true if the driver is viable, false
 if it is not. The second value should be an explanation as to why the driver is
 not viable (in cases where it is not).
 
+=item $string = Your::Driver::version_string()
+
+=item $string = Your::Driver::version_string(\%PARAMS)
+
+=item $string = Your::Driver->version_string()
+
+=item $string = Your::Driver->version_string(\%PARAMS)
+
+=item $string = $db->version_string()
+
+=item $string = $db->version_string(\%PARAMS)
+
+The default implementation returns 'unknown'.
+
+This is complicated because it can be called as a function, a class method, or
+an object method. It can also optionally be called with a hashref of PARAMS
+that MAY be later used to construct an instance.
+
+Lets assume your driver uses the C<start_my_db> command to launch a database.
+Normally you default to the C<start_my_db> found in the $PATH environment
+variable. Alternatively someone can pass in an alternative path to the binary
+with the 'launcher' parameter. Here is a good implementation:
+
+    use Scalar::Util qw/reftype/;
+
+    sub version_string {
+        my $binary;
+
+        # Go in reverse order assuming the last param hash provided is most important
+        for my $arg (reverse @_) {
+            my $type = reftype($arg) or next; # skip if not a ref
+            next $type eq 'HASH'; # We have a hashref, possibly blessed
+
+            # If we find a launcher we are done looping, we want to use this binary.
+            $binary = $arg->{launcher} and last;
+        }
+
+        # If no args provided one to use we fallback to the default from $PATH
+        $binary ||= DEFAULT_BINARY;
+
+        # Call the binary with '-V', capturing and returning the output using backticks.
+        return `$binary -V`;
+    }
+
 =item $socket = $db->socket()
 
 Unix Socket used to communicate with the db. If the db type does not use
@@ -610,7 +763,7 @@ F<https://github.com/exodist/DBIx-QuickDB/>.
 
 =head1 COPYRIGHT
 
-Copyright 2018 Chad Granum E<lt>exodist7@gmail.comE<gt>.
+Copyright 2020 Chad Granum E<lt>exodist7@gmail.comE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
