@@ -2,9 +2,11 @@ package DBIx::QuickDB::Driver::MySQL;
 use strict;
 use warnings;
 
-our $VERSION = '0.000010';
+our $VERSION = '0.000011';
 
 use IPC::Cmd qw/can_run/;
+use DBIx::QuickDB::Util qw/strip_hash_defaults/;
+use Scalar::Util qw/reftype/;
 
 use parent 'DBIx::QuickDB::Driver';
 
@@ -26,6 +28,25 @@ BEGIN {
     $DBDMYSQL = eval { require DBD::mysql; 'DBD::mysql' };
 }
 
+sub version_string {
+    my $binary;
+
+    # Go in reverse order assuming the last param hash provided is most important
+    for my $arg (reverse @_) {
+        my $type = reftype($arg) or next;    # skip if not a ref
+        next $type eq 'HASH';                # We have a hashref, possibly blessed
+
+        # If we find a launcher we are done looping, we want to use this binary.
+        $binary = $arg->{+MYSQLD} and last;
+    }
+
+    # If no args provided one to use we fallback to the default from $PATH
+    $binary ||= $MYSQLD;
+
+    # Call the binary with '-V', capturing and returning the output using backticks.
+    return `$binary -V`;
+}
+
 sub list_env_vars {
     my $self = shift;
     return (
@@ -166,6 +187,31 @@ sub init {
     }
 }
 
+sub clone {
+    my $self = shift;
+    my $clone = $self->SUPER::clone(@_);
+
+
+    return $clone;
+}
+
+sub clone_data {
+    my $self = shift;
+
+    my $config = strip_hash_defaults(
+        $self->{+CONFIG},
+        { $self->_default_config },
+    );
+
+    return (
+        $self->SUPER::clone_data(),
+
+        CONFIG() => $config,
+        MYSQLD() => $self->{+MYSQLD},
+        MYSQL()  => $self->{+MYSQL},
+    );
+}
+
 sub write_config {
     my $self = shift;
     my (%params) = @_;
@@ -312,7 +358,7 @@ F<https://github.com/exodist/DBIx-QuickDB/>.
 
 =head1 COPYRIGHT
 
-Copyright 2018 Chad Granum E<lt>exodist7@gmail.comE<gt>.
+Copyright 2020 Chad Granum E<lt>exodist7@gmail.comE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
