@@ -2,9 +2,12 @@ package DBIx::QuickDB::Driver::PostgreSQL;
 use strict;
 use warnings;
 
-our $VERSION = '0.000010';
+our $VERSION = '0.000011';
 
 use IPC::Cmd qw/can_run/;
+use DBIx::QuickDB::Util qw/strip_hash_defaults/;
+use Time::HiRes qw/sleep/;
+use Scalar::Util qw/reftype/;
 
 use parent 'DBIx::QuickDB::Driver';
 
@@ -30,6 +33,25 @@ BEGIN {
     $DBDPG    = eval { require DBD::Pg; 'DBD::Pg'};
 }
 
+sub version_string {
+    my $binary;
+
+    # Go in reverse order assuming the last param hash provided is most important
+    for my $arg (reverse @_) {
+        my $type = reftype($arg) or next;    # skip if not a ref
+        next if $type eq 'HASH';             # We have a hashref, possibly blessed
+
+        # If we find a launcher we are done looping, we want to use this binary.
+        $binary = $arg->{+POSTGRES} and last;
+    }
+
+    # If no args provided one to use we fallback to the default from $PATH
+    $binary ||= $POSTGRES;
+
+    # Call the binary with '-V', capturing and returning the output using backticks.
+    return `$binary -V`;
+}
+
 sub list_env_vars {
     my $self = shift;
     return (
@@ -128,14 +150,28 @@ sub init {
     }
 }
 
-sub bootstrap {
+sub clone_data {
     my $self = shift;
 
-    my $dir = $self->{+DIR};
-    my $db_dir = $self->{+DATA_DIR};
-    mkdir($db_dir) or die "Could not create data dir: $!";
-    $self->run_command([$self->{+INITDB}, '-E', 'UTF8', '-D', $db_dir]);
+    my $vars = $self->env_vars || {};
+    delete $vars->{PGPORT} if $vars->{PGPORT} && $vars->{PGPORT} eq $self->port;
 
+    my $config = strip_hash_defaults(
+        $self->{+CONFIG},
+        { $self->_default_config },
+    );
+
+    return (
+        $self->SUPER::clone_data(),
+        ENV_VARS() => $vars,
+        CONFIG()   => $config,
+    );
+}
+
+sub write_config {
+    my $self = shift;
+
+    my $db_dir = $self->{+DATA_DIR};
     open(my $cf, '>', "$db_dir/postgresql.conf") or die "Could not open config file: $!";
     for my $key (sort keys %{$self->{+CONFIG}}) {
         my $val = $self->{+CONFIG}->{$key};
@@ -144,7 +180,17 @@ sub bootstrap {
         print $cf "$key = $val\n";
     }
     close($cf);
+}
 
+sub bootstrap {
+    my $self = shift;
+
+    my $dir = $self->{+DIR};
+    my $db_dir = $self->{+DATA_DIR};
+    mkdir($db_dir) or die "Could not create data dir: $!";
+    $self->run_command([$self->{+INITDB}, '-E', 'UTF8', '-D', $db_dir]);
+
+    $self->write_config;
     $self->start;
 
     for my $try ( 1 .. 5 ) {
@@ -160,6 +206,33 @@ sub bootstrap {
     return;
 }
 
+sub connect {
+    my $self = shift;
+    my ($db_name, %params) = @_;
+
+    my $dbh;
+    my $start = time;
+    while (1) {
+        my $waited = time - $start;
+        die "Timeout waiting for server" if $waited > 10;
+
+        my ($ok, $err);
+        {
+            local $@;
+            $ok = eval {
+                $dbh = $self->SUPER::connect($db_name, %params);
+                1;
+            };
+
+            $err = $@;
+        }
+
+        return $dbh if $ok;
+        die $err unless $err =~ m/the database system is starting up/;
+        sleep 0.01;
+    }
+}
+
 sub connect_string {
     my $self = shift;
     my ($db_name) = @_;
@@ -241,7 +314,7 @@ F<https://github.com/exodist/DBIx-QuickDB/>.
 
 =head1 COPYRIGHT
 
-Copyright 2018 Chad Granum E<lt>exodist7@gmail.comE<gt>.
+Copyright 2020 Chad Granum E<lt>exodist7@gmail.comE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
