@@ -2,7 +2,7 @@ package Test2::Tools::QuickDB;
 use strict;
 use warnings;
 
-our $VERSION = '0.000010';
+our $VERSION = '0.000011';
 
 use Carp qw/croak/;
 use Test2::API qw/context/;
@@ -41,10 +41,12 @@ sub skipall_unless_can_db {
 
     my $drivers = $spec{driver} ? [$spec{driver}] : $spec{drivers} || [DBIx::QuickDB->plugins];
 
+    my $reason;
     my $ok = 0;
     for my $driver (@$drivers) {
         next unless defined $driver;
         my ($v, $fqn, $why) = DBIx::QuickDB->check_driver($driver, \%spec);
+        $reason = $why if @$drivers == 1;
         next unless $v;
         $ok = $fqn;
         last;
@@ -55,7 +57,7 @@ sub skipall_unless_can_db {
         return $ok;
     }
 
-    $ctx->plan(0, 'SKIP' => "no db driver is viable");
+    $ctx->plan(0, 'SKIP' => $reason || "no db driver is viable");
     $ctx->release;
 
     return;
@@ -192,7 +194,7 @@ F<https://github.com/exodist/DBIx-QuickDB/>.
 
 =head1 COPYRIGHT
 
-Copyright 2018 Chad Granum E<lt>exodist7@gmail.comE<gt>.
+Copyright 2020 Chad Granum E<lt>exodist7@gmail.comE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
